﻿using System;
using CommonBehaviors.Actions;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

using L = PrivDH.Core.Utilities.Log;

namespace PrivDH.Core
{
    static class Spell
    {
        public delegate WoWUnit unitSelectionDelegate(object context);

        public delegate T Selection<out T>(object context);

        public static int lastSpellCast;
        private static LocalPlayer Me => StyxWoW.Me;
        private static WoWUnit currentTarget => StyxWoW.Me.CurrentTarget;

        #region [Method] - Cast Method

        private static Composite castMethod(this int Spell, unitSelectionDelegate unitSelection,
            Selection<bool> reqs = null, bool gcdCast = false)
        {
            return
                new Decorator(
                    ret =>
                        ((reqs != null && reqs(ret)) || (reqs == null)) && unitSelection != null &&
                        unitSelection(ret) != null && SpellManager.CanCast(Spell, unitSelection(ret)),
                    new Action(ret =>
                    {
                        if (SpellManager.Cast(Spell, unitSelection(ret)))
                        {
                            lastSpellCast = Spell;
                            L.combatLog("Casting: " + WoWSpell.FromId(Spell).Name + " on " + unitSelection(ret).SafeName);
                            if (!gcdCast)
                                return RunStatus.Success;
                        }
                        return RunStatus.Failure;
                    }));
        }

        #endregion

        #region Drop Method

        private static Composite dropMethod(this int Spell, unitSelectionDelegate unitSelection, Selection<bool> reqs = null, bool waitForSpell = false)
        {
            return new Decorator(ret => unitSelection != null && reqs(ret) && SpellManager.CanCast(Spell) && (Me.Location.Distance(unitSelection(ret).Location) <= WoWSpell.FromId(Spell).MaxRange || WoWSpell.FromId(Spell).MaxRange == 0),
            new Sequence(
            new Action(ret => SpellManager.Cast(Spell)),
            new DecoratorContinue(ctx => waitForSpell,
            new WaitContinue(1, ret => Me.CurrentPendingCursorSpell != null && Me.CurrentPendingCursorSpell.Id == Spell, new ActionAlwaysSucceed())),
            new Action(ret =>
            {
                SpellManager.ClickRemoteLocation(unitSelection(ret).Location);
                lastSpellCast = Spell;
                L.combatLog("Casting: " + Spell);
            })));
        }
        #endregion

        #region [Method] - Cooldown Detection

        public static double cooldownTimeLeft(this int Spell)
        {
            try
            {
                SpellFindResults Results;
                return SpellManager.FindSpell(Spell, out Results)
                    ? (Results.Override != null
                        ? Results.Override.CooldownTimeLeft.TotalMilliseconds
                        : Results.Original.CooldownTimeLeft.TotalMilliseconds)
                    : 9999;
            }
            catch (Exception xException)
            {
                L.diagnosticLog("Exception in cooldownTimeLeft(): ", xException);
                return 9999;
            }
        }

        #endregion

        #region [Method] - Ground Casting
        public static Composite dropCast(this int Spell, unitSelectionDelegate unitSelection, Selection<bool> reqs = null)
        {
            return dropMethod(Spell, unitSelection, reqs);
        }
        #endregion

        #region [Method] - Standard Casting

        public static Composite Buff(this int Spell, Selection<bool> reqs = null, bool gcdCast = false)
        {
            return castMethod(Spell, ret => Me, reqs, gcdCast);
        }

        public static Composite Cast(this int Spell, Selection<bool> reqs = null, bool gcdCast = false)
        {
            return castMethod(Spell, ret => currentTarget, reqs, gcdCast);
        }

        #endregion

        #region Spell Charges

        public static int GetCharges(string name)
        {
            SpellFindResults sfr;
            if (SpellManager.FindSpell(name, out sfr))
            {
                WoWSpell spell = sfr.Override ?? sfr.Original;
                return GetCharges(spell);
            }
            return 0;
        }

        public static int GetCharges(WoWSpell spell)
        {
            int charges = Lua.GetReturnVal<int>("return GetSpellCharges(" + spell.Id.ToString() + ")", 0);
            return charges;
        }
        #endregion
    }
}
