﻿using System;
using System.Linq;
using System.Numerics;
using Styx;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System.Collections.Generic;
using System.Threading.Tasks;

using PrivDH.Core.Helpers;



using L = PrivDH.Core.Utilities.Log;
using Styx.Common;

namespace PrivDH.Core
{
    static class Unit
    {
        private static LocalPlayer Me => StyxWoW.Me;

        #region [Method] - Aura Detection

        public static bool auraOnMe;
        public static bool auraOnTarget;
        public static uint auraStackOnMe;
        public static uint auraStackOnTarget;
        public static uint mainResource;

        public static bool auraExists(this WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return false;
                WoWAura Aura = isMyAura
                    ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid)
                    : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null;
            }
            catch (Exception xException)
            {
                L.diagnosticLog("Exception in auraExists(): ", xException);
                return false;
            }
        }

        public static uint auraStacks(this WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return 0;
                WoWAura Aura = isMyAura ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid) : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null ? Aura.StackCount : 0;
            }
            catch (Exception xException)
            {
                L.diagnosticLog("Exception in auraStacks(): ", xException);
                return 0;
            }
        }

        public static double auraTimeLeft(this WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return 0;
                WoWAura Aura = isMyAura ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid) : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null ? Aura.TimeLeft.TotalMilliseconds : 0;
            }
            catch (Exception xException)
            {
                L.diagnosticLog("Exception in auraTimeLeft(): ", xException);
                return 9999;
            }
        }

        #endregion

        #region [Method] - Cache
        public static void Cache()
        {
            mainResource = Me.CurrentFury;
        }
        #endregion

        #region [Method] - Unit Evaluation

        public static bool isUnitValid(this WoWUnit Unit, double Range)
        {
            return Unit != null && Unit.IsValid && Unit.IsAlive && Unit.Attackable && Unit.DistanceSqr <= Range * Range;
        }

        public static bool isAttackable(WoWUnit unit)
        {
            if (unit == null)
                return false;
            if (!unit.CanSelect || !unit.Attackable)
                return false;
            if (unit.IsFriendly)
                return false;
            if (unit.IsDead)
                return false;
            if (unit.IsOnTransport)
                return false;
            if (unit.IsPet || unit.OwnedByRoot != null)
                return false;
            if (unit.IsNonCombatPet || unit.IsCritter)
                return false;
            if (unit.IsTotem)
                return false;
            return true;

        }

        #endregion



        #region [Methods] - Nearby Units

        private static List<WoWUnit> nearbyUnits(Vector3 fromVector3, double radius, bool playerOnly)
        {
            List<WoWUnit> hostile = ObjectManager.GetObjectsOfType<WoWUnit>(true, false);
            var maxDistance2 = radius * radius;
            if (playerOnly)
            {
                hostile =
                    hostile.Where(x => x.IsPlayer && isAttackable(x) && x.Location.Distance(fromVector3) < maxDistance2)
                        .ToList();
            }
            else
            {
                hostile =
                    hostile.Where(x => !x.IsPlayer && isAttackable(x) && x.Location.Distance(fromVector3) < maxDistance2)
                        .ToList();
            }
            return hostile;
        }

        public static int CountEnemiesInRange(Vector3 fomVector3, double maxRange)
        {
            return nearbyUnits(fomVector3, maxRange, PrivDH.LocationContext == GroupLogic.Battleground).Count;
        }
        #endregion
    }
}
